@@ -30,6 +30,9 @@
 #include "core/utils.h"
 #include "guiutils.h"
 #include "settings.h"
+#include "core/observer.h"
+#include "core/tile.h"
+#include "settings_core.h"
 
 K_GLOBAL_STATIC_WITH_ARGS( QPixmap, busyPixmap, ( KIconLoader::global()->loadIcon("okular", KIconLoader::NoGroup, 32, KIconLoader::DefaultState, QStringList(), 0, true) ) )
 
@@ -63,46 +66,54 @@ void PagePainter::paintCroppedPageOnPainter( QPainter * destPainter, const Okula
 	int croppedWidth = scaledCrop.width();
 	int croppedHeight = scaledCrop.height();
 
-    /** 1 - RETRIEVE THE 'PAGE+ID' PIXMAP OR A SIMILAR 'PAGE' ONE **/
-    const QPixmap * pixmap = page->_o_nearestPixmap( pixID, scaledWidth, scaledHeight );
-
-    QColor color = Qt::white;
-    if ( Okular::Settings::changeColors() )
+    QColor paperColor = Qt::white;
+    QColor backgroundColor = paperColor;
+    if ( Okular::SettingsCore::changeColors() )
     {
-        switch ( Okular::Settings::renderMode() )
+        switch ( Okular::SettingsCore::renderMode() )
         {
-            case Okular::Settings::EnumRenderMode::Inverted:
-                color = Qt::black;
+            case Okular::SettingsCore::EnumRenderMode::Inverted:
+                backgroundColor = Qt::black;
                 break;
-            case Okular::Settings::EnumRenderMode::Paper:
-                color = Okular::Settings::paperColor();
+            case Okular::SettingsCore::EnumRenderMode::Paper:
+                paperColor = Okular::SettingsCore::paperColor();
+                backgroundColor = paperColor;
                 break;
-            case Okular::Settings::EnumRenderMode::Recolor:
-                color = Okular::Settings::recolorBackground();
+            case Okular::SettingsCore::EnumRenderMode::Recolor:
+                backgroundColor = Okular::Settings::recolorBackground();
                 break;
             default: ;
         }
     }
-    destPainter->fillRect( limits, color );
+    destPainter->fillRect( limits, backgroundColor );
 
-    /** 1B - IF NO PIXMAP, DRAW EMPTY PAGE **/
-    double pixmapRescaleRatio = pixmap ? scaledWidth / (double)pixmap->width() : -1;
-    long pixmapPixels = pixmap ? (long)pixmap->width() * (long)pixmap->height() : 0;
-    if ( !pixmap || pixmapRescaleRatio > 20.0 || pixmapRescaleRatio < 0.25 ||
-         (scaledWidth != pixmap->width() && pixmapPixels > 6000000L) )
+    const bool hasTilesManager = ( pixID == PAGEVIEW_ID && page->hasTilesManager() );
+    const QPixmap *pixmap = 0;
+
+    if ( !hasTilesManager )
     {
-        // draw something on the blank page: the okular icon or a cross (as a fallback)
-        if ( !busyPixmap->isNull() )
+        /** 1 - RETRIEVE THE 'PAGE+ID' PIXMAP OR A SIMILAR 'PAGE' ONE **/
+        pixmap = page->_o_nearestPixmap( pixID, scaledWidth, scaledHeight );
+
+        /** 1B - IF NO PIXMAP, DRAW EMPTY PAGE **/
+        double pixmapRescaleRatio = pixmap ? scaledWidth / (double)pixmap->width() : -1;
+        long pixmapPixels = pixmap ? (long)pixmap->width() * (long)pixmap->height() : 0;
+        if ( !pixmap || pixmapRescaleRatio > 20.0 || pixmapRescaleRatio < 0.25 ||
+             (scaledWidth != pixmap->width() && pixmapPixels > 6000000L) )
         {
-            destPainter->drawPixmap( QPoint( 10, 10 ), *busyPixmap );
+            // draw something on the blank page: the okular icon or a cross (as a fallback)
+            if ( !busyPixmap->isNull() )
+            {
+                destPainter->drawPixmap( QPoint( 10, 10 ), *busyPixmap );
+            }
+            else
+            {
+                destPainter->setPen( Qt::gray );
+                destPainter->drawLine( 0, 0, croppedWidth-1, croppedHeight-1 );
+                destPainter->drawLine( 0, croppedHeight-1, croppedWidth-1, 0 );
+            }
+            return;
         }
-        else
-        {
-            destPainter->setPen( Qt::gray );
-            destPainter->drawLine( 0, 0, croppedWidth-1, croppedHeight-1 );
-            destPainter->drawLine( 0, croppedHeight-1, croppedWidth-1, 0 );
-        }
-        return;
     }
 
     /** 2 - FIND OUT WHAT TO PAINT (Flags + Configuration + Presence) **/
@@ -220,27 +231,51 @@ void PagePainter::paintCroppedPageOnPainter( QPainter * destPainter, const Okula
     }
 
     /** 3 - ENABLE BACKBUFFERING IF DIRECT IMAGE MANIPULATION IS NEEDED **/
-    bool bufferAccessibility = (flags & Accessibility) && Okular::Settings::changeColors() && (Okular::Settings::renderMode() != Okular::Settings::EnumRenderMode::Paper);
+    bool bufferAccessibility = (flags & Accessibility) && Okular::SettingsCore::changeColors() && (Okular::SettingsCore::renderMode() != Okular::SettingsCore::EnumRenderMode::Paper);
     bool useBackBuffer = bufferAccessibility || bufferedHighlights || bufferedAnnotations || viewPortPoint;
     QPixmap * backPixmap = 0;
     QPainter * mixedPainter = 0;
-    QRect limitsInPixmap = limits.translated( crop.geometry( scaledWidth, scaledHeight ).topLeft() );
+    QRect limitsInPixmap = limits.translated( scaledCrop.topLeft() );
         // limits within full (scaled but uncropped) pixmap
 
     /** 4A -- REGULAR FLOW. PAINT PIXMAP NORMAL OR RESCALED USING GIVEN QPAINTER **/
     if ( !useBackBuffer )
     {
-        // 4A.1. if size is ok, draw the page pixmap using painter
-        if ( pixmap->width() == scaledWidth && pixmap->height() == scaledHeight )
-            destPainter->drawPixmap( limits.topLeft(), *pixmap, limitsInPixmap );
-
-        // else draw a scaled portion of the magnified pixmap
+        if ( hasTilesManager )
+        {
+            const Okular::NormalizedRect normalizedLimits( limitsInPixmap, scaledWidth, scaledHeight );
+            const QList<Okular::Tile> tiles = page->tilesAt( normalizedLimits );
+            QList<Okular::Tile>::const_iterator tIt = tiles.constBegin(), tEnd = tiles.constEnd();
+            while ( tIt != tEnd )
+            {
+                const Okular::Tile &tile = *tIt;
+                QRect tileRect = tile.rect().geometry( scaledWidth, scaledHeight ).translated( -scaledCrop.topLeft() );
+                QRect limitsInTile = limits & tileRect;
+                if ( !limitsInTile.isEmpty() )
+                {
+                    if ( tile.pixmap()->width() == tileRect.width() && tile.pixmap()->height() == tileRect.height() )
+                        destPainter->drawPixmap( limitsInTile.topLeft(), *(tile.pixmap()),
+                                limitsInTile.translated( -tileRect.topLeft() ) );
+                    else
+                        destPainter->drawPixmap( tileRect, *(tile.pixmap()) );
+                }
+                tIt++;
+            }
+        }
         else
         {
-            QImage destImage;
-            scalePixmapOnImage( destImage, pixmap, scaledWidth, scaledHeight, limitsInPixmap );
-            destPainter->drawImage( limits.left(), limits.top(), destImage, 0, 0,
-                                     limits.width(),limits.height() );
+            // 4A.1. if size is ok, draw the page pixmap using painter
+            if ( pixmap->width() == scaledWidth && pixmap->height() == scaledHeight )
+                destPainter->drawPixmap( limits.topLeft(), *pixmap, limitsInPixmap );
+
+            // else draw a scaled portion of the magnified pixmap
+            else
+            {
+                QImage destImage;
+                scalePixmapOnImage( destImage, pixmap, scaledWidth, scaledHeight, limitsInPixmap );
+                destPainter->drawImage( limits.left(), limits.top(), destImage, 0, 0,
+                                         limits.width(),limits.height() );
+            }
         }
 
         // 4A.2. active painter is the one passed to this method
@@ -251,28 +286,72 @@ void PagePainter::paintCroppedPageOnPainter( QPainter * destPainter, const Okula
     {
         // the image over which we are going to draw
         QImage backImage;
-        bool has_alpha = pixmap->hasAlpha();
 
-        // 4B.1. draw the page pixmap: normal or scaled
-        if ( pixmap->width() == scaledWidth && pixmap->height() == scaledHeight )
-            cropPixmapOnImage( backImage, pixmap, limitsInPixmap );
+        bool has_alpha;
+        if ( pixmap )
+            has_alpha = pixmap->hasAlpha();
         else
-            scalePixmapOnImage( backImage, pixmap, scaledWidth, scaledHeight, limitsInPixmap );
+            has_alpha = true;
+
+        if ( hasTilesManager )
+        {
+            backImage = QImage( limits.width(), limits.height(), QImage::Format_ARGB32_Premultiplied );
+            backImage.fill( paperColor.rgb() );
+            QPainter p( &backImage );
+            const Okular::NormalizedRect normalizedLimits( limitsInPixmap, scaledWidth, scaledHeight );
+            const QList<Okular::Tile> tiles = page->tilesAt( normalizedLimits );
+            QList<Okular::Tile>::const_iterator tIt = tiles.constBegin(), tEnd = tiles.constEnd();
+            while ( tIt != tEnd )
+            {
+                const Okular::Tile &tile = *tIt;
+                QRect tileRect = tile.rect().geometry( scaledWidth, scaledHeight ).translated( -scaledCrop.topLeft() );
+                QRect limitsInTile = limits & tileRect;
+                if ( !limitsInTile.isEmpty() )
+                {
+                    if ( !tile.pixmap()->hasAlpha() )
+                        has_alpha = false;
+
+                    if ( tile.pixmap()->width() == tileRect.width() && tile.pixmap()->height() == tileRect.height() )
+                    {
+                        p.drawPixmap( limitsInTile.translated( -limits.topLeft() ).topLeft(), *(tile.pixmap()),
+                                limitsInTile.translated( -tileRect.topLeft() ) );
+                    }
+                    else
+                    {
+                        double xScale = tile.pixmap()->width() / (double)tileRect.width();
+                        double yScale = tile.pixmap()->height() / (double)tileRect.height();
+                        QTransform transform( xScale, 0, 0, yScale, 0, 0 );
+                        p.drawPixmap( limitsInTile.translated( -limits.topLeft() ), *(tile.pixmap()),
+                                transform.mapRect( limitsInTile ).translated( -transform.mapRect( tileRect ).topLeft() ) );
+                    }
+                }
+                ++tIt;
+            }
+            p.end();
+        }
+        else
+        {
+            // 4B.1. draw the page pixmap: normal or scaled
+            if ( pixmap->width() == scaledWidth && pixmap->height() == scaledHeight )
+                cropPixmapOnImage( backImage, pixmap, limitsInPixmap );
+            else
+                scalePixmapOnImage( backImage, pixmap, scaledWidth, scaledHeight, limitsInPixmap );
+        }
 
         // 4B.2. modify pixmap following accessibility settings
         if ( bufferAccessibility )
         {
-            switch ( Okular::Settings::renderMode() )
+            switch ( Okular::SettingsCore::renderMode() )
             {
-                case Okular::Settings::EnumRenderMode::Inverted:
+                case Okular::SettingsCore::EnumRenderMode::Inverted:
                     // Invert image pixels using QImage internal function
                     backImage.invertPixels(QImage::InvertRgb);
                     break;
-                case Okular::Settings::EnumRenderMode::Recolor:
+                case Okular::SettingsCore::EnumRenderMode::Recolor:
                     // Recolor image using Blitz::flatten with dither:0
                     Blitz::flatten( backImage, Okular::Settings::recolorForeground(), Okular::Settings::recolorBackground() );
                     break;
-                case Okular::Settings::EnumRenderMode::BlackWhite:
+                case Okular::SettingsCore::EnumRenderMode::BlackWhite:
                     // Manual Gray and Contrast
                     unsigned int * data = (unsigned int *)backImage.bits();
                     int val, pixels = backImage.width() * backImage.height(),
